using System;
using System.Drawing;
using System.Numerics;
using SlatedGameToolkit.Framework.Graphics.Textures;

namespace SlatedGameToolkit.Framework.Graphics.Render
{
    public class RectangleMesh : IMesh
    {
        private Matrix4x4 matRot = Matrix4x4.Identity;
        private bool changed;
        private Vector3 rotation;
        private Vector2 origin, dimensions;
        public readonly Vector2[] textureCoords = new Vector2[4];
        public ValueTuple<Vector3, Vector2>[] vertices = new ValueTuple<Vector3, Vector2>[4];
        private uint[] indices = new uint[] {0, 1, 3, 1, 2, 3};

        public ValueTuple<Vector3, Vector2>[] Vertices
        {
            get {
                if (changed) CalculateVertices();
                return vertices;
            }
        }

        public float X {
            get 
            {
                return origin.X;
            }

            set 
            {
                changed = true;
                origin.X = value;
            }
        }

        public float Y {
            get
            {
                return origin.Y;
            }

            set
            {
                changed = true;
                origin.Y = value;
            }
        }

        public float Width
        {
            get
            {
                return dimensions.X;
            }

            set
            {
                changed = true;
                dimensions.X = value;
            }
        }

        public float Height
        {
            get
            {
                return dimensions.Y;
            }

            
            set
            {
                changed = true;
                dimensions.Y = value;
            }
        }

        public uint[] Elements { get {return indices; } }

        public Vector3 Rotation 
        {
            get 
            {
                return rotation;
            }

            set
            {
                changed = true;
                rotation = value;
                matRot = Matrix4x4.CreateFromYawPitchRoll(value.X, value.Y, value.Z);
            }
        }

        public ITexture Texture { get; set; }

        public Color Color { get; set; }

        public RectangleMesh(ITexture texture, Color color)
        {
            this.Texture = texture;
            this.Color = color;

            this.textureCoords[0].X = 0;
            this.textureCoords[0].Y = 1;

            this.textureCoords[1].X = 1;
            this.textureCoords[1].Y = 1;
            
            this.textureCoords[2].X = 1;
            this.textureCoords[2].Y = 0;
            
            this.textureCoords[3].X = 0;
            this.textureCoords[3].Y = 0;
        }

        
        private void CalculateVertices() {
            if (!changed) return;
            Vector3[] baseVerts = new Vector3[4];
            baseVerts[0] = new Vector3(this.origin, 0);
            baseVerts[1] = new Vector3(this.origin.X + this.dimensions.X, this.origin.Y, 0);
            baseVerts[2] = new Vector3(baseVerts[1].X, this.origin.Y + this.dimensions.Y, 0);
            baseVerts[3] = new Vector3(this.origin.X, baseVerts[2].Y, 0);

            for (int i = 0; i < vertices.Length; i++)
            {
                vertices[i] = (Vector3.Transform(baseVerts[i], matRot), textureCoords[i]);
            }
            changed = false;
        }
    }
}